@@ -20,7 +20,7 @@ import re
 
 import gtk
 from gtk import gdk
-import gnomevfs
+import gio
 import gedit
 
 from Library import Library
@@ -113,9 +113,15 @@ class Document:
 
                         self.update_language()
                 elif self.language_id != 0:
-                        Library().unref(self.language_id)
-                        self.language_id = 0
-        
+                        langid = self.language_id
+                        
+                        self.language_id = None;
+                        
+                        if self.instance:
+                                self.instance.language_changed(self)
+
+                        Library().unref(langid)
+
         def set_view(self, view):
                 if view == self.view:
                         return
@@ -132,8 +138,7 @@ class Document:
                 elif lang and lang.get_id() == self.language_id:
                         return
 
-                if self.language_id != 0:
-                        Library().unref(self.language_id)
+                langid = self.language_id
 
                 if lang:
                         self.language_id = lang.get_id()
@@ -143,6 +148,9 @@ class Document:
                 if self.instance:
                         self.instance.language_changed(self)
 
+                if langid != 0:
+                        Library().unref(langid)
+
                 Library().ref(self.language_id)
 
         def accelerator_activate(self, keyval, mod):
@@ -460,7 +468,7 @@ class Document:
                                                 placeholder[0].leave()
                                                 
                                         remove.append(placeholder)
-                
+
                 for placeholder in remove:
                         if placeholder == self.active_placeholder:
                                 self.active_placeholder = None
@@ -620,7 +628,8 @@ class Document:
                 current = self.current_placeholder()
                 
                 if current:
-                        self.update_placeholders.append(current)
+                        if not current in self.update_placeholders:
+                                self.update_placeholders.append(current)
                 
                         if self.timeout_update_id != 0:
                                 gobject.source_remove(self.timeout_update_id)
@@ -693,9 +702,6 @@ class Document:
                 if len(path1) - 1 > 3:
                         return second
                 
-                print path1
-                print path2
-                
                 if mime.startswith('x-directory'):
                         # directory, special case
                         if not path2:
@@ -717,7 +723,8 @@ class Document:
         def apply_uri_snippet(self, snippet, mime, uri):
                 # Remove file scheme
                 if gedit.utils.uri_has_file_scheme(uri):
-                        uri = gnomevfs.get_local_path_from_uri(uri)
+                        gfile = gio.File(uri)
+                        uri = gfile.get_path()
                 
                 # Set environmental variables
                 filename = self.env_get_filename(self.view.get_buffer())
@@ -751,7 +758,7 @@ class Document:
                 
                 for uri in uris:
                         try:
-                                mime = gnomevfs.get_mime_type(uri)
+                                mime = gio.content_type_guess(uri)
                         except:
                                 mime = None
 
